@@ -25,11 +25,7 @@
 #include <libusb-1.0/libusb.h>
 
       int VENDOR_ID = ProgramOptions::Instance().getVendorUSB_ID();
-const int VENDOR_ID_GRAPHTEC = 0x0b4d; 
       int PRODUCT_ID = ProgramOptions::Instance().getProductUSB_ID();
-const int PRODUCT_ID_CC200_20 = 0x110a;
-const int PRODUCT_ID_SILHOUETTE_SD_1 = 0x111c;
-const int PRODUCT_ID_SILHOUETTE_SD_2 = 0x111d;
 
 #include <iostream>
 #include <cmath>
@@ -87,10 +83,10 @@ Error UsbSend(libusb_device_handle* handle, const string& s, int timeout = 0)
 			cerr << "Error writing to device: " << UsbError(ret) << endl;
 			return Error("Error writing to device: " + UsbError(ret));
 		}
-		if (transferred != data.length())
+		if ((unsigned int)transferred != data.length())
 		{
 			cerr << "Warning, some data not transferred correctly." << endl;
-			return Error("Some data not transfered. Attempted: " + ItoS(data.length()) + " Transferred: " + ItoS(transferred));
+			return Error("Some data not transferred. Attempted: " + ItoS(data.length()) + " Transferred: " + ItoS(transferred));
 		}
 	}
 	return Success;
@@ -127,33 +123,15 @@ Error UsbReceive(libusb_device_handle* handle, string& s, int timeout = 0)
 //		unsigned char *data, uint16_t length, unsigned int timeout);
 //}
 
-Error Cut(QList<QPolygonF> cuts, double mediawidth, double mediaheight, int media, int speed, int pressure, bool trackenhancing,
-		  bool regmark, bool regsearch, double regwidth, double reglength)
+libusb_device_handle *UsbOpen(struct cutter_id *id)
 {
-	VENDOR_ID = ProgramOptions::Instance().getVendorUSB_ID();
-	PRODUCT_ID = ProgramOptions::Instance().getProductUSB_ID();
-	
-	cout << "Cutting... VENDOR_ID : " << VENDOR_ID << " PRODUCT_ID: " << PRODUCT_ID
-		 << " mediawidth: " << mediawidth << " mediaheight: " << mediaheight
-		 << "media: " << media << " speed: " << speed << " pressure: " << pressure
-		 << " trackenhancing: " << trackenhancing << " regmark: " <<  regmark
-		 << " regsearch:" <<  regsearch <<" regwidth:" <<  regwidth << " reglength: " << reglength << endl;
-
-	if (media < 100 || media > 300)
-		media = 300;
-	if (speed < 1)
-		speed = 1;
-	if (speed > 10)
-		speed = 10;
-	if (pressure < 1)
-		pressure = 1;
-	if (pressure > 33)
-		pressure = 33;
-
 	libusb_device** list;
 	ssize_t cnt = libusb_get_device_list(NULL, &list);
 	if (cnt < 1)
-		return Error("Couldn't get usb device list.");
+	{
+		id->msg = Error("Couldn't get usb device list.");
+		return NULL;
+	}
 
 	int err = 0;
 
@@ -164,11 +142,21 @@ Error Cut(QList<QPolygonF> cuts, double mediawidth, double mediaheight, int medi
 		libusb_device_descriptor desc;
 		libusb_get_device_descriptor(device, &desc);
 		// I don't want to be more specific than this really.
-		if ((desc.idVendor == VENDOR_ID || desc.idVendor == VENDOR_ID_GRAPHTEC) &&
-				(desc.idProduct == PRODUCT_ID || desc.idProduct == PRODUCT_ID_CC200_20 || desc.idProduct == PRODUCT_ID_SILHOUETTE_SD_1 || desc.idProduct == PRODUCT_ID_SILHOUETTE_SD_2))
+		if ((desc.idVendor == VENDOR_ID || desc.idVendor == VENDOR_ID_GRAPHTEC) && 
+		    (desc.idProduct == PRODUCT_ID ||
+		     desc.idProduct == PRODUCT_ID_CC200_20 ||
+		     desc.idProduct == PRODUCT_ID_CC300_20 ||
+		     desc.idProduct == PRODUCT_ID_SILHOUETTE_SD_1 ||
+		     desc.idProduct == PRODUCT_ID_SILHOUETTE_SD_2 ||
+		     desc.idProduct == PRODUCT_ID_SILHOUETTE_CAMEO ||
+		     desc.idProduct == PRODUCT_ID_SILHOUETTE_PORTRAIT
+		     )
+		    )
 		{
 			// Just use the first one. Who has two?!
 			found = device;
+			id->usb_product_id = desc.idProduct;
+			id->usb_vendor_id = desc.idVendor;
 			break;
 		}
 	}
@@ -176,7 +164,8 @@ Error Cut(QList<QPolygonF> cuts, double mediawidth, double mediaheight, int medi
 	if (!found)
 	{
 		libusb_free_device_list(list, 1);
-		return Error("Couldn't find Craft Robo 2 (USB device with Vendor ID with USB VendorID 0b4d and ProductID 110a). Is it connected to the system and powered on?");
+		id->msg = Error("Couldn't find Craft Robo 2 (USB device with Vendor ID with USB VendorID 0b4d and ProductID 110a). Is it connected to the system and powered on?");
+		return NULL;
 	}
 
 	libusb_device_handle* handle;
@@ -185,21 +174,33 @@ Error Cut(QList<QPolygonF> cuts, double mediawidth, double mediaheight, int medi
 	if (err != 0)
 	{
 		libusb_free_device_list(list, 1);
-		return Error("Error accessing Craft Robo 2: " + UsbError(err) + ". Do you have permission (on Linux make sure you are in the group 'lp').");
+		id->msg = Error("Error accessing Craft Robo 2: " + UsbError(err) + ". Do you have permission (on Linux make sure you are in the group 'lp').");
+		return NULL;
 	}
 
 	libusb_free_device_list(list, 1);
+	id->msg = "Ready";
+	return handle;
+}
 
+// caller can use UsbSend() afterwards, and should
+// finally do libusb_release_interface(handle, 0); libusb_close(handle);
+libusb_device_handle *UsbInit(struct cutter_id *id)
+{
 	// Now do stuff with the handle.
 	int r = 0;
 
+	libusb_device_handle* handle = UsbOpen(id);
+	if (!handle) return NULL;
+
 	if (libusb_kernel_driver_active(handle, 0) == 1)
 	{
 		r = libusb_detach_kernel_driver(handle, 0);
 		if (r != 0)
 		{
 			libusb_close(handle);
-			return Error("Error detaching kernel USB driver: " + UsbError(r));
+			id->msg = Error("Error detaching kernel USB driver: " + UsbError(r));
+			return NULL;
 		}
 	}
 
@@ -207,7 +208,8 @@ Error Cut(QList<QPolygonF> cuts, double mediawidth, double mediaheight, int medi
 	if (r != 0)
 	{
 		libusb_close(handle);
-		return Error("Error resetting device: " + UsbError(r));
+		id->msg = Error("Error resetting device: " + UsbError(r));
+		return NULL;
 	}
 
 	cout << "Selecting configuration." << endl;
@@ -215,7 +217,8 @@ Error Cut(QList<QPolygonF> cuts, double mediawidth, double mediaheight, int medi
 	if (r < 0)
 	{
 		libusb_close(handle);
-		return Error("Error setting USB configuration: " + UsbError(r));
+		id->msg = Error("Error setting USB configuration: " + UsbError(r));
+		return NULL;
 	}
 
 
@@ -224,18 +227,113 @@ Error Cut(QList<QPolygonF> cuts, double mediawidth, double mediaheight, int medi
 	if (r < 0)
 	{
 		libusb_close(handle);
-		return Error("Error claiming USB interface: " + UsbError(r));
+		id->msg = Error("Error claiming USB interface: " + UsbError(r));
+		return NULL;
 	}
 
 	cout << "Setting alt interface." << endl;
 	r = libusb_set_interface_alt_setting(handle, 0, 0); // Probably not really necessary.
 	if (r < 0)
 	{
-		return Error("Error setting alternate USB interface: " + UsbError(r));
+		libusb_close(handle);
+		id->msg = Error("Error setting alternate USB interface: " + UsbError(r));
+		return NULL;
 	}
 
 	cout << "Initialisation successful." << endl;
+	return handle;
+}
 
+struct cutter_id *Identify()
+{
+	static struct cutter_id id = { "?", 0, 0 };
+	libusb_device_handle *handle;
+
+	if (1)
+	  {
+	    handle = UsbOpen(&id);
+	    if (handle)
+	      libusb_close(handle);
+	    else
+	      id.msg = "no device found";
+	  }
+	else
+	  {
+	    id.msg = Error("Cannot Identify while cut thread is running");
+	  }
+	return &id;
+}
+
+
+QList<QPolygonF> Transform_Silhouette_Cameo(QList<QPolygonF> cuts, double *mediawidth, double *mediaheight)
+{
+	const double devicewidth = 300;		// cameo is 12inch aka 300mm wide
+	double w;
+	double h = *mediaheight;
+	
+	cout << "Transform_Silhouette_Cameo " << *mediawidth << "," << *mediaheight << endl;
+	if (0) 
+	{
+		w = *mediawidth;
+		cout << "Paper right aligned" << endl;
+	}
+	else
+	{
+		w = devicewidth;
+		// adjust the used media area, so that the hardware does not clip.
+		*mediawidth = devicewidth;
+		cout << "Paper left aligned" << endl;
+	}
+	  
+
+	// flip it around 180 deg, and go backwards through the path list.
+	QList<QPolygonF> paths;
+  	for (int i = cuts.size()-1; i >= 0; i--)
+	{
+		QPolygonF poly;
+		for (int j = 0; j < cuts[i].size(); j++)
+		{
+			// QPolygonF is a QList<QPointF>
+			double x = cuts[i][j].x();
+			double y = cuts[i][j].y();
+			poly << QPointF(w-x, h-y);
+		}
+		paths << poly;
+	}
+  return paths;
+}
+
+Error Cut(CutParams p)
+{
+	VENDOR_ID = ProgramOptions::Instance().getVendorUSB_ID();
+	PRODUCT_ID = ProgramOptions::Instance().getProductUSB_ID();
+	
+	cout << "Cutting... VENDOR_ID : " << VENDOR_ID << " PRODUCT_ID: " << PRODUCT_ID
+		 << " mediawidth: " << p.mediawidth << " mediaheight: " << p.mediaheight
+		 << "media: " << p.media << " speed: " << p.speed << " pressure: " << p.pressure
+		 << " trackenhancing: " << p.trackenhancing << " regmark: " <<  p.regmark
+		 << " regsearch:" <<  p.regsearch <<" regwidth:" <<  p.regwidth << " reglength: " << p.regheight << endl;
+
+	if (p.media < 100 || p.media > 300)
+		p.media = 300;
+	if (p.speed < 1)
+		p.speed = 1;
+	if (p.speed > 10)
+		p.speed = 10;
+	if (p.pressure < 1)
+		p.pressure = 1;
+	if (p.pressure > 33)
+		p.pressure = 33;
+
+	// how can VENDOR_ID / PRODUCT_ID report the correct values abve???
+	struct cutter_id id = { "?", 0, 0 };
+	libusb_device_handle* handle = UsbInit(&id);
+	if (id.usb_vendor_id == VENDOR_ID_GRAPHTEC &&
+	   id.usb_product_id == PRODUCT_ID_SILHOUETTE_CAMEO)
+	  {
+	    // should this also transform the regwidth regheigth or not?
+		p.cuts = Transform_Silhouette_Cameo(p.cuts, &p.mediawidth, &p.mediaheight);
+	  }
 
 	// TODO: Use exceptions.
 
@@ -253,9 +351,14 @@ Error Cut(QList<QPolygonF> cuts, double mediawidth, double mediaheight, int medi
 	e = UsbReceive(handle, resp, 5000);
 	if (!e) goto error;
 
-	if (resp != "0\x03") // 0 = Ready. 1 = Moving. "  " = ??
+	if (resp != "0\x03") // 0 = Ready. 1 = Moving. 2 = Nothing loaded. "  " = ??
 	{
-		e = Error("Invalid response from plotter: " + resp);
+		if (resp == "1\x03")
+		  e = Error("Moving, please try again.");
+		else if (resp == "2\x03")
+		  e = Error("Empty tray, please load media.");	// Silhouette Cameo
+		else
+		  e = Error("Invalid response from plotter: " + resp);
 		goto error;
 	}
 
@@ -277,23 +380,23 @@ Error Cut(QList<QPolygonF> cuts, double mediawidth, double mediaheight, int medi
 //		goto error;
 //	}
 
-	e = UsbSend(handle, "FW" + ItoS(media) + "\x03");
+	e = UsbSend(handle, "FW" + ItoS(p.media) + "\x03");
 	if (!e) goto error;
 
-	e = UsbSend(handle, "!" + ItoS(speed) + "\x03");
+	e = UsbSend(handle, "!" + ItoS(p.speed) + "\x03");
 	if (!e) goto error;
 
-	e = UsbSend(handle, "FX" + ItoS(pressure) + "\x03");
+	e = UsbSend(handle, "FX" + ItoS(p.pressure) + "\x03");
 	if (!e) goto error;
 
 	// I think this sets the distance from the position of the plotter
 	// head to the actual cutting point, maybe in 0.1 mms (todo: Measure blade).
 	// It is 0 for the pen, 18 for cutting.
 	// C possible stands for curvature. Not that any of the other letters make sense...
-	e = UsbSend(handle, "FC" + ItoS(media == 113 ? 0 : 18) + "\x03");
+	e = UsbSend(handle, "FC" + ItoS(p.media == 113 ? 0 : 18) + "\x03");
 	if (!e) goto error;
 
-	e = UsbSend(handle, "FY" + ItoS(trackenhancing ? 0 : 1) + "\x03");
+	e = UsbSend(handle, "FY" + ItoS(p.trackenhancing ? 0 : 1) + "\x03");
 	if (!e) goto error;
 
 	// Set to portrait. FN1 does landscape but it's easier just to rotate the image.
@@ -327,8 +430,8 @@ Error Cut(QList<QPolygonF> cuts, double mediawidth, double mediaheight, int medi
 		// Page size: height,width in 20ths of a mm minus a margin. This is for A4. TODO: Find maximum and use that.
 		stringstream page;
 
-		int width = lroundl(mediawidth * 20.0);
-		int height = lroundl(mediaheight * 20.0);
+		int width = lroundl(p.mediawidth * 20.0);
+		int height = lroundl(p.mediaheight * 20.0);
 
 		int margintop = ProgramOptions::Instance().getMarginTop();
 		int marginright = ProgramOptions::Instance().getMarginRight();
@@ -337,18 +440,18 @@ Error Cut(QList<QPolygonF> cuts, double mediawidth, double mediaheight, int medi
 		if (!e) goto error;
 		e = UsbSend(handle, "FM1\x03"); // ?
 		if (!e) goto error;
-		
-		if (regmark)
+
+		if (p.regmark)
 		{
 			stringstream regmarkstr;
 			regmarkstr.precision(0);
 			string searchregchar = "23,";
-			int regw = lroundl(regwidth * 20.0);
-			int regl = lroundl(reglength * 20.0);
+			int regw = lroundl(p.regwidth * 20.0);
+			int regl = lroundl(p.regheight * 20.0);
 			e = UsbSend(handle, "TB50,381\x03"); //only with registration (it was TB50,1) ???
 			if (!e) goto error;
 			
-			if (regsearch)
+			if (p.regsearch)
 				searchregchar ="123,";
 			
 			regmarkstr <<  "TB99\x03TB55,1\x03TB" + searchregchar + ItoS(regw) + "," + ItoS(regl) + "\x03";
@@ -393,7 +496,7 @@ Error Cut(QList<QPolygonF> cuts, double mediawidth, double mediaheight, int medi
 			e = UsbSend(handle, "TB50,1\x03"); // ???
 			if (!e) goto error;
 		}
- 
+
 		if (!e) goto error;
 		// I think this is the feed command. Sometimes it is 5588 - maybe a maximum?
 		e = UsbSend(handle, "FO" + ItoS(height - margintop) + "\x03");
@@ -402,19 +505,18 @@ Error Cut(QList<QPolygonF> cuts, double mediawidth, double mediaheight, int medi
 		page.flags(ios::fixed);
 		page.precision(0);
 		page << "&100,100,100,\\0,0,Z" << ItoS(width) << "," << ItoS(height) << ",L0";
-
-		for (int i = 0; i < cuts.size(); ++i)
+		for (int i = 0; i < p.cuts.size(); ++i)
 		{
-			if (cuts[i].size() < 2)
+			if (p.cuts[i].size() < 2)
 				continue;
 
-			double x = cuts[i][0].x()*20.0;
-			double y = cuts[i][0].y()*20.0;
+			double x = p.cuts[i][0].x()*20.0;
+			double y = p.cuts[i][0].y()*20.0;
 
 			double xorigin = ProgramOptions::Instance().getRegOriginWidthMM();
 			double yorigin = ProgramOptions::Instance().getRegOriginHeightMM();
 
-			if (regmark)
+			if (p.regmark)
 			{
 				x = x - (xorigin*20.0);
 				y = y + (yorigin*20.0);
@@ -428,12 +530,12 @@ Error Cut(QList<QPolygonF> cuts, double mediawidth, double mediaheight, int medi
 			if (y > height) y = height;
 
 			page << ",M" << x << "," << height-y;
-			for (int j = 1; j < cuts[i].size(); ++j)
+			for (int j = 1; j < p.cuts[i].size(); ++j)
 			{
-				x = cuts[i][j].x()*20.0;
-				y = cuts[i][j].y()*20.0;
+				x = p.cuts[i][j].x()*20.0;
+				y = p.cuts[i][j].y()*20.0;
 
-				if (regmark)
+				if (p.regmark)
 				{
 					x = x - (xorigin*20.0);
 					y = y + (yorigin*20.0);
@@ -466,9 +568,10 @@ Error Cut(QList<QPolygonF> cuts, double mediawidth, double mediaheight, int medi
 			}
 		}
 
+
 		page << "&1,1,1,TB50,0\x03"; // TB maybe .. ah I dunno. Need to experiment. No idea what &1,1,1 does either.
 
-		cout << page.str() << endl;
+		// cout << page.str() << endl;
 
 		e = UsbSend(handle, page.str());
 		if (!e) goto error;
@@ -490,6 +593,7 @@ Error Cut(QList<QPolygonF> cuts, double mediawidth, double mediaheight, int medi
 	return Success;
 
 error: // Hey, this is basically C and I can't be bothered to properly C++-ify it. TODO: Use exceptions.
+        cout << "Error: " << e << endl;
 	libusb_release_interface(handle, 0);
 	libusb_close(handle);
 	return e;
